Package: glmnet
Type: Package
Title: Lasso and Elastic-Net Regularized Generalized Linear Models
Version: 5.0
Date: 2026-04-27
Authors@R: c(person("Jerome", "Friedman", role=c("aut")),
	     person("Trevor", "Hastie", role=c("aut", "cre"), email = "hastie@stanford.edu"),
	     person("Rob", "Tibshirani", role=c("aut")),
	     person("Balasubramanian", "Narasimhan", role=c("aut")),
	     person("Kenneth","Tay",role=c("aut")),
	     person("Noah", "Simon", role=c("aut")),
	     person("Junyang", "Qian", role=c("ctb")),
	     person("James", "Yang", role=c("aut")),
	     person("Jonathan","Taylor", role=c("aut")))
Depends: R (>= 3.6.0), Matrix (>= 1.0-6)
Imports: methods, utils, foreach, shape, survival, Rcpp
Suggests: knitr, lars, nnet, testthat, xfun, rmarkdown
SystemRequirements: C++17
Description: Extremely efficient procedures for fitting the entire lasso or elastic-net regularization path for linear regression, logistic and multinomial regression models, Poisson regression, Cox model,  multiple-response Gaussian, and the grouped multinomial regression; see <doi:10.18637/jss.v033.i01> and <doi:10.18637/jss.v039.i05>. There are two new and important additions. The family argument can be a GLM family object, which opens the door to any programmed family (<doi:10.18637/jss.v106.i01>). This comes with a modest computational cost, so when the built-in families suffice, they should be used instead.  The other novelty is the relax option, which refits each of the active sets in the path unpenalized. The algorithm uses cyclical coordinate descent in a path-wise fashion, as described in the papers cited.
License: GPL-2
VignetteBuilder: knitr
Encoding: UTF-8
URL: https://glmnet.stanford.edu
RoxygenNote: 7.3.3
LinkingTo: RcppEigen, Rcpp
NeedsCompilation: yes
Packaged: 2026-04-30 21:22:11 UTC; hastie
Author: Jerome Friedman [aut],
  Trevor Hastie [aut, cre],
  Rob Tibshirani [aut],
  Balasubramanian Narasimhan [aut],
  Kenneth Tay [aut],
  Noah Simon [aut],
  Junyang Qian [ctb],
  James Yang [aut],
  Jonathan Taylor [aut]
Maintainer: Trevor Hastie <hastie@stanford.edu>
Repository: CRAN
Date/Publication: 2026-05-04 05:40:42 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-04 06:39:27 UTC; unix
Archs: glmnet.so.dSYM
