# Generated by roxygen2: do not edit by hand

S3method("[",stratifySurv)
S3method(buildPredmat,array)
S3method(buildPredmat,coxnetlist)
S3method(buildPredmat,default)
S3method(buildPredmat,glmnetfitlist)
S3method(buildPredmat,lognetlist)
S3method(buildPredmat,mrelnetlist)
S3method(buildPredmat,multnetlist)
S3method(coef,cv.glmnet)
S3method(coef,cv.relaxed)
S3method(coef,glmnet)
S3method(coef,relaxed)
S3method(deviance,glmnet)
S3method(family,cv.glmnet)
S3method(family,glmnet)
S3method(family,glmnetfit)
S3method(family,relaxed)
S3method(plot,cv.glmnet)
S3method(plot,cv.relaxed)
S3method(plot,glmnet)
S3method(plot,mrelnet)
S3method(plot,multnet)
S3method(plot,relaxed)
S3method(predict,coxnet)
S3method(predict,cv.glmnet)
S3method(predict,cv.relaxed)
S3method(predict,elnet)
S3method(predict,fishnet)
S3method(predict,glmnet)
S3method(predict,glmnetfit)
S3method(predict,lognet)
S3method(predict,mrelnet)
S3method(predict,multnet)
S3method(predict,relaxed)
S3method(print,bigGlm)
S3method(print,confusion.table)
S3method(print,cv.glmnet)
S3method(print,cv.relaxed)
S3method(print,glmnet)
S3method(print,relaxed)
S3method(survfit,coxnet)
S3method(survfit,cv.glmnet)
export(Cindex)
export(assess.glmnet)
export(bigGlm)
export(buildPredmat)
export(coef.glmnet)
export(coef.relaxed)
export(confusion.glmnet)
export(coxgrad)
export(coxnet.deviance)
export(cv.glmnet)
export(glmnet)
export(glmnet.control)
export(glmnet.measures)
export(makeX)
export(na.replace)
export(na_sparse_fix)
export(predict.glmnet)
export(predict.relaxed)
export(prepareX)
export(print.cv.glmnet)
export(relax.glmnet)
export(rmult)
export(roc.glmnet)
export(stratifySurv)
import(Matrix)
import(foreach)
import(methods)
importFrom(Matrix,sparse.model.matrix)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(shape,colorlegend)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,contrasts)
importFrom(stats,family)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,glm.fit)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.pass)
importFrom(stats,predict)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,weighted.mean)
importFrom(survival,Surv)
importFrom(survival,concordance)
importFrom(survival,coxph)
importFrom(survival,is.Surv)
importFrom(survival,strata)
importFrom(survival,survfit)
importFrom(utils,modifyList)
importFrom(utils,packageDescription)
useDynLib(glmnet)
